@@ -33,7 +33,7 @@ struct value_pair {
 };
 
 #define OPT_LEN_MAX 	4096
-#define PARSE_MAX_VP	16
+#define PARSE_MAX_VP	24
 
 /*
  * Option define
@@ -46,7 +46,9 @@ struct fio_option {
 	unsigned int off2;
 	unsigned int off3;
 	unsigned int off4;
-	void *roff1, *roff2, *roff3, *roff4;
+	unsigned int off5;
+	unsigned int off6;
+	void *roff1, *roff2, *roff3, *roff4, *roff5, *roff6;
 	unsigned int maxval;		/* max and min value */
 	int minval;
 	double maxfp;			/* max and min floating value */
@@ -65,7 +67,7 @@ struct fio_option {
 
 typedef int (str_cb_fn)(void *, char *);
 
-extern int parse_option(const char *, struct fio_option *, void *);
+extern int parse_option(char *, const char *, struct fio_option *, struct fio_option **, void *);
 extern void sort_options(char **, struct fio_option *, int);
 extern int parse_cmd_option(const char *t, const char *l, struct fio_option *, void *);
 extern int show_cmd_help(struct fio_option *, const char *);
