@@ -36,7 +36,7 @@ struct fio_net_int_cmd {
 };
 
 enum {
-	FIO_SERVER_VER		= 6,
+	FIO_SERVER_VER		= 8,
 
 	FIO_SERVER_MAX_PDU	= 1024,
 
@@ -78,9 +78,7 @@ struct cmd_du_pdu {
 struct cmd_probe_pdu {
 	uint8_t hostname[64];
 	uint8_t bigendian;
-	uint8_t fio_major;
-	uint8_t fio_minor;
-	uint8_t fio_patch;
+	uint8_t fio_version[32];
 	uint8_t os;
 	uint8_t arch;
 	uint8_t bpp;
@@ -126,6 +124,7 @@ extern int fio_clients_send_ini(const char *);
 extern int fio_handle_clients(void);
 extern int fio_client_add(const char *, void **);
 extern void fio_client_add_cmd_option(void *, const char *);
+extern void fio_client_add_ini_file(void *, const char *);
 
 extern int fio_recv_data(int sk, void *p, unsigned int len);
 extern int fio_send_data(int sk, const void *p, unsigned int len);
